function [sigma, gamma, itr] = eval_mu(A, B, alpha, beta, tol)
% [sigma, gamma, itr] = eval_mu(lambda, A, B) computes
%                max_gamma sigma_2k-1  [A-alpha*B, -beta*gamma*B]
%                                      [beta/gamma*B A-alpha*B  ]
%   where lambda = alpha + beta*i, A and B be n*k matrices.
%   gamma in (0,1]. tol is the accuracy tolerance for gamma
%   i.e., |gamma-gamma*| < tol.
%
%   Valid for both dense and sparse inputs.
%
%   Use golden section search, combined with parabolic interpolation,
%   see MATLAB function fminbnd.
%
%   OUTPUT:   
%       optimal parameter gamma
%       optimal singular value sigma
%       number of total iterations itr

%   Revised: Jun. 13, 2016.

if nargin < 5
    tol = 1.0E-6;
end

f = @(g) -second_smallest_sv(A, B, alpha, beta, g);

if beta==0
    sigma = -f(1);
    gamma = .5; % any gamma in (0,1] is a maximizer
    itr = 1;
    return;
end

options.TolX = tol;
[gamma, sigma, ~, output] = fminbnd(f, 0, 1, options);
sigma = -sigma;
itr = output.iterations;


