function [alphatt, betatt, gammatt, utol, conv] = rpsa_global_verify(A, B, epsln, alphat, ctol, utol)
%   Check if the vertical line of alphat + ctol*abs(alphat) intersects
%   the real epsln pseudospectrum of (A,B).
%
%   Use recurrent scheme for small ctol, i.e., v-search with 
%       alpha + utol, alpha + utol/10, ... alpha + ctol.
%   conv = false, if maxit is reached without achieving convergence  

%   Revised: Jun. 13, 2016.

vtol = eps;
while true 
    alpha_check = alphat + utol*abs(alphat);
    [betatt, gammatt, conv] = Vertical_Search(A, B, alpha_check,...
                                 epsln, vtol);
    if betatt == -inf 
        if utol > ctol % v-search with utol fail, reduce utol
            utol = max([utol/10,ctol]); 
        else % v-search fail with ctol, return alphat
            alphatt = alphat;
            break;
        end
    else
        alphatt = alpha_check; % return the intersection points
        break;
    end
end

return
  



% ================================================================ 
% Subfunctions
% ================================================================ 

function [beta, gamma, conv] = Vertical_Search(A, B, alpha0, epsln, tol)
%   vertical search (monotonic reduction) starting with alpha0 + beta0*j.
%   To find the `uppermost' vertical intersection with the real pseudospectrum.
%
%   INPUT:
%       alpha0  vertical search level
%       epsln   perturbation level
%       tol     the (relative) tolerance s.t. 
%               |eval_mu(alpha + j*beta0) - epsln|  < epsln*tol
%               default by 1.0E-6
%
%   OUTPUT:
%       beta    imag part of intersection point
%               beta = -inf, if no intersection was found. 
%       gamma   touching superset parameter
%       conv    true, if converged 
%
%   POSSIBLE IMPROVEMENT: update the convergence test, line 66
%

maxit = 60;
gamma = .1; % initial gamma
beta00 = -inf;
conv = true;

% Compute initial upper bound: the uppermost intersection
% with superset of gamma.
[reg,~] = superset_eig_imag(A, B, alpha0, [], gamma, epsln); 
beta = max(reg);

% Monotonic reduction 
for i = 1 : maxit
    % vertical search found no intersection 
    if isempty(beta)
        beta = -inf; itr = i; return;
    end

    % update gamma
    [epsln_relax, gamma] = eval_mu(A,B,alpha0,beta,tol);

    % convergence check
    if epsln_relax - epsln < epsln*tol || abs(beta - beta00) < abs(beta)*tol 
        itr = i;
        return;
    end

    % update beta
    beta00 = beta; 
    [reg,~] = superset_eig_imag(A, B, alpha0, [], gamma, epsln);
    beta = max( reg( reg<beta00 + abs(beta00)*1.0E-6 ) );
end

if isempty(beta)
    beta = -inf;
end

% maxit reached 
conv = false;

return
% END
