function [alpha, gamma, itr, info, hstr] = rpsa_h_search(A, B, alpha0, beta0, epsln, tol, dodisp)
%   Horizontal search with monotonic reduction starting at alpha0 + beta0*j
%
%   INPUT:
%       alpha0,beta0    starting value (real and imag part)
%                       if alpha0 == [], then use random gamma for initial superset.
%       epsln           perturbation level
%       tol             the (relative) tolerance s.t. 
%                       |mu(alpha + j*beta0) - epsln|  < epsln*tol
%                       default by 1.0E-6
%       dodisp          if true, then display convergence history.
%
%   OUTPUT:
%       alpha   real part of intersection point
%               alpha = -inf, if starting value lies outside the pseudospectrum.
%       gamma   touching superset parameter at (alpha, beta0)
%       itr     number of iterations.
%       info    infomation flag
%               info = 1    success
%               info = 2    starting point outside the pseudospectrum
%               info = 3    maximum iteration number (default by 30) reached
%               info = 4    Horizontal search converged with relative increment in alpha
%                           close to machine precision.
%                           The solution may not satisfy the prescribed tolerance.
%                           This is possibly caused by the numerical error in the
%                           eigenvalue computation, and mu value computation.
%       hstr    convergence history of alpha
%
%   POSSIBLE IMPROVEMENT: update the convergence test, line 66
%

%   Revised: Jun. 13, 2016.

if nargin < 7, dodisp = false; end
if nargin < 6, tol = 1.0E-6; end 
maxit = 60;
info = 1;
mu_tol = eps; 

% Compute the initial gamma and mu level based on (alpha0, beta0).
if isempty(alpha0)
    gamma = 0.5;
    alpha0 = -inf;
else
    [~, gamma] = eval_mu(A, B, alpha0, beta0, mu_tol);
end
alpha00 = alpha0;
hstr = [alpha0, 0];

% Compute initial upper bound: the rightmost intersection with superset of gamma.
alpha = superset_eig_real(A, B, alpha0, beta0, gamma, epsln); 
alpha = max([alpha; alpha0]);

% Monotonic reduction 
for i = 1 : maxit
    if isempty(alpha) % no intersection found
        alpha = -inf;
        itr = i;
        info = 4;
        break;
    else
        hstr = [hstr; alpha, gamma];
    end

    % update gamma
    [epsln_relax, gamma] = eval_mu(A, B, alpha, beta0, mu_tol);
    
    % stopping criteria: updating step too small
    if i > 1 && alpha00 - alpha < abs(alpha00) * eps
        info = 4;
        itr = i;
        break;
    end

    % stopping criteria: mu value converged
    if epsln_relax - epsln < epsln * tol 
        itr = i;
        break;
    end

    % update alpha: rightmost intersection with the superset of gamma.
    %               inside (alpha0, alpha)
    alpha00 = alpha; 
    reg = superset_eig_real(A, B, alpha0, beta0, gamma, epsln);
    alpha = max( reg( reg < (alpha00 + abs(alpha00) * 1.0E-6)) );
    alpha = min([alpha, alpha00]);
end

if dodisp % display convergence history
    disp(' ');
    disp(['          *** Convergence of H-searh start at (', ...
            num2str(hstr(1,1)),',' num2str(beta0),') *** ']);
    fprintf('%9s  %18s  %9s  \r\n', 'k', 'alpha_k', 'gamma_k');
    for i = 1:length(hstr)
        fprintf('%9i  %+16.15f  %8.7f  \n', [i-1, hstr(i,:)]);
    end
    disp('          *********************************************** ');
    disp(' ');
end

% maxit reached 
if i == maxit 
    itr = maxit;
    info = 3;
end

return;
% END
