function [alpha, beta, gamma, info] = rpsa_pencil(A, B, epsln, opts)
%
%   [alpha, beta, gamma] = rpsa_pencil(A, B, epsln) solves the rightmost 
%       point alpha + beta * j in the (generalized) real pseudospectrum
%       Lambda_epsln(A,B) of a (rectangular) matrix pencil A-z*B, where
%       gamma = gamma_max(alpha,beta).
%
%   INPUT:
%       A, B:   real matrices of size n*p, p<n.
%       epsln:  perturbation level.
%
%       opts:   optional parameters
%       - opts.maxit (60)       maximum iterations.
%       - opts.tol (1.0E-8)     stopping (relative) increment tolerance
%       - opts.ctol (1.0E-2)    global verification tolerance
%       - opts.dodisp (false)   whether to display computation history
%       - opts.histp (false)    use history points to divide interval
%       - opts.initck (true)    use user provided initial value, if false 
%       - opts.do_acc (false)   use Aitken acceleration applied to gamma
%
%   OUTPUT:
%       alpha, beta, gamma:     real and imaginary part of the rightmost 
%                               point, gamma = gamma_max(alpha,beta).
%
%       info:           information structures
%       - info.its      total number of iterations (number of horizontal search).
%       - info.hstr     computation history, each row 
%                       [alpha_i, beta_i, gamma_i, isvred, ...
%                                       lb0, ub0, lb1, ub1, isgck].
%                       for i = 0 : itr, and i is the number of horizontal
%                       search.
%                       [lb0, ub0] is the maximizing interval,
%                       [lb1, ub1] is the reduced interval, if 
%                           isvred = true, with [lb0, ub0] to be the 
%                           reducing interval
%                       isvred, isgck, indicates whether v-reduce and 
%                       global check is performed
%       - info.succ     true if successful, otherwise false
%       - info.message  stopping message
%       
%   References: 
%       [1] Ding Lu and Bart Vandereycken, Criss-cross type algorithms 
%           for computing the real pseudospectral abscissa,
%           manuscript available at http://www.unige.ch/math/vandereycken.
%
%   Authors:
%       Ding Lu and Bart Vandereycken, Univ. of Geneva.
%
%   Version:
%       0.1, Jun. 13, 2016.
%

% PARAMETER CHECK
if nargin < 4, opts = []; end
[maxit, tol, ctol, dodisp, histp, alpha0, beta0, initck, do_acc] = Check_Opts(opts);

% INILIZATION
htol = tol;                 % horizontal search tol, not recommend to use eps
                            % H-search should be slightly larger...
utol = max([ctol, 1.0E-3]); % iterative global verification tol
sz = size(A);
if sz(1) >= 2 * sz(2)       % Dimension reduction for the rectangular case
    [~, R] = qr([B, A], 0);
    B = R(:, 1:sz(2));
    A = R(:, sz(2)+1:end);
end
if isempty(alpha0)          % initialize starting `eigenvalue'
    [alpha0, beta0] = rpsa_init_eigval(A, B, epsln);
elseif initck
    [alpha00, beta00] = rpsa_init_eigval(A, B, epsln);
    if alpha0 < alpha00
        alpha0 = alpha00;
        beta0 = beta00;
    end
end
if isempty(alpha0)          % Failed to find a starting point, quick return 
    alpha = NaN; beta = NaN; gamma = NaN;
    info.succ = false;
    info.message = ' Fail to find an initial point to start criss-cross search ';
    return;
end

hstr = [alpha0, beta0, 0, 0, -inf, inf, -inf, inf, 0]; 

% FIRST HORIZONTAL SEARCH 
[alpha, gamma, ~] = rpsa_h_search(A, B, alpha0, beta0, epsln, htol);
beta = beta0;
isred = 0; 
isgck = 0; 
hstr = [hstr; 
        alpha, beta, gamma, isred, -inf, inf, -inf, inf, isgck]; 

if dodisp
    Display_Result(0, [], []); % print title line
    Display_Result(2, hstr, 0);% print first two lines
end

%   MAIN LOOP
for i = 1 : maxit
    % COMPUTE MAXIMIZING INTERVALS
    [~, itvv] = superset_eig_imag(A, B, alpha, beta, gamma, epsln);
    itv0 = itvv;

    % INTERVAL EXTENSION BY HISTORY POINTS (optional) 
    if histp, itvv = Interval_Extend(itvv, hstr(:,2)); end

    if ~isempty(itvv)   % midpoints 
        npt = .5 * ( itvv(:,1) + itvv(:,2) );
    else
        npt = [];
    end
     
    % HORIZONTAL SEARCH
    alphat = -inf; 
    betat  = -inf; 
    gammat = -inf;    
    maxitv_idx = -inf; 
    maxitv = [-inf, inf, -inf, inf]; 
    maxitv_info = true;
    for j = 1 : length(npt) 
        betac = npt(j);
        [alpha00, gamma00, itrr, info_h, ~] =...
                rpsa_h_search(A, B, alpha, betac, epsln, htol);
        if alpha00 >= alphat    % update by larger real part
            alphat = alpha00; betat  = betac; gammat = gamma00;
            maxitv_idx = j;
            maxitv_info = (info_h~=3); % successful h search 
        end 
    end
    if ~isinf(maxitv_idx)       % record maximizing interval
        maxitv = [itvv(maxitv_idx, :), -inf, inf];
    end

    % SAFEGUARDED INTERVAL REDUCTION
    if alphat - alpha < eps * abs(alpha) 
        [alphat, betat, gammat, isred, maxitv] = ...
                rpsa_v_reduce(A, B, epsln, alpha, beta, gamma, itv0, htol);
    end

    % CONVERGENCE CHECK: alpha updates too small, or large number of its reached
    if alphat- alpha< tol * abs(alpha) || i == round(maxit/2)   
        % Global verification 
        [alphatt, betatt, gammatt, utol, conv] = ...
                rpsa_global_verify(A, B, epsln, alphat, ctol, utol);
        if betatt == -inf && conv   % No future intersection found
            alpha = alphat; beta = betat; gamma = gammat; isgck = 1;
            hstr = [hstr; alpha, beta, gamma, isred, maxitv, isgck];
            break;
        elseif conv                % Intersection found 
            alphat = alphatt; betat = betatt; gammat = gammatt; isgck = 1;
        else                       % Global verification not converged
            alpha = alphat; beta = betat; gamma = gammat; isgck = 1;
            hstr = [hstr; alpha, beta, gamma, isred, maxitv, isgck];
            break;
        end
    end;


    % PREPARE FOR NEXT ITERATION
    alpha = alphat; 
    beta = betat;
    gamma = gammat;
    if do_acc && mod(i, 3) == 0 %Aitken acceleration
        gamma1 = hstr(end-1, 3);
        gamma2 = hstr(end, 3);
        gammat = (gamma*gamma1 - gamma2^2) / (gamma1+gamma - 2*gamma2); 
        if gammat > 0 && gammat < 1, gamma = gammat;end;
    end

    hstr = [hstr; 
            alpha, beta, gamma, isred, maxitv, isgck];

    if dodisp, Display_Result(1, hstr(end, :), i + 1); end

    isred = 0;
    isgck = 0;
end

if isempty(alpha)
    alpha = NaN; beta = NaN; gamma = NaN;
end

% RETURN INFO STRUCTURE
info = Check_Convergence(i, maxit, conv, hstr, maxitv_info);

if dodisp 
    if info.succ, Display_Result( 1, hstr(end, :), i + 1); end
    Display_Result( inf, hstr, []); % plot bottom line
end

% END
return;




% ================================================================ 
% Subfunctions
% ================================================================ 

function [maxit, tol, ctol, dodisp, histp, alpha0, beta0, initck, do_acc] = Check_Opts(opts)
if isfield(opts, 'maxit') && ~isempty(opts.maxit) && opts.maxit > 0   % max iteration number
    maxit = opts.maxit;
else
    maxit = 60;
end
if isfield(opts, 'tol') && ~isempty(opts.tol) && opts.tol > 0    % stopping criteria, increment tolerance
    tol = opts.tol;
else
    tol = 1.0E-8;
end
if isfield(opts, 'ctol') && ~isempty(opts.ctol) && opts.ctol > 0   % global verification tolerance
    ctol = opts.ctol;
else
    ctol = 1.0E-2;
end
if isfield(opts, 'dodisp') && ~isempty(opts.dodisp)  % display convergence history 
    dodisp = opts.dodisp;
else
    dodisp = false;
end
if isfield(opts, 'histp') && ~isempty(opts.histp)   % history points check
    histp = opts.histp;
else
    histp = false;
end
if isfield(opts,'alpha0') && ~isempty(opts.alpha0) &&  ...
    isfield(opts, 'beta0') && ~isempty(opts.beta0) % user provided 
    alpha0 = opts.alpha0; beta0 = opts.beta0;
else 
    alpha0 = [];
    beta0 = [];
end
if isfield(opts, 'initck') && ~isempty(opts.initck)   % initial value check
    initck = opts.initck;
else
    initck = true;
end
if isfield(opts, 'do_acc') && ~isempty(opts.do_acc)   % initial value check
    do_acc = opts.do_acc;
else
    do_acc = false;
end
return


% -----------------------------------------------------------------

function itvv = Interval_Extend(itvv, hstr)
% Extend and split the maximizing intervals by the imaginary parts 
%   of the computed points.
%   Each row of itvv is an candidate interval.
%   Each element of hstr is an candidate history point.

kk = size(itvv,1); % number of intervals
i = length(hstr); % number of history points
for ii = 1 : i
    betat = hstr(ii);
    for jj = 1:kk
        if betat > itvv(jj,1) && betat < itvv(jj,2)
            % split the interval when the history point falls inside. 
            itvv = [itvv; betat, itvv(jj,2)];
            itvv(jj,2) = betat;
            kk = kk + 1; 
            break;
        end
    end
end

return;


% -----------------------------------------------------------------

function Display_Result(mode, hstr, itr)
% Display convergence history of criss-cross
%  mode = 0: print title line 
%  mode = N: print n rows in hstr, start with iteration number itr
%  mode = inf: print bottom line

if mode == 0 
    fprintf('       *******************************************************\n');
    fprintf('       *** Criss-Cross Convergence ***\n');
    fprintf('%9s  %22s  %13s  %9s  %6s \n', 'k', 'alpha_k', 'beta_k', 'gamma_k', 'V-Red?');
elseif ~isinf(mode)
    for i = 1 : mode
        fprintf('%9i  %+16.15e  %8.7e  %8.7f  %3s\n', ...
            [itr + i - 1, hstr(i,1:3)], log2str(hstr(i,4)) );
    end
else
    fprintf('       ***\n       Iterations %2i,  V-Reduce %2i,  Global-Check %2i\n ', ... 
         size(hstr, 1) - 1, nnz( hstr(:,4) ), nnz(hstr(:, end)) );
    fprintf('      *******************************************************\n\n');
end;
return;

function s = log2str(x)
if x 
    s = 'Y';
else
    s = 'N';
end

return

% -----------------------------------------------------------------

function info = Check_Convergence(i, maxit, conv, hstr, maxitv_info)

info.hstr = hstr; 
info.its = i + 1;
if i~=maxit && conv && maxitv_info
    info.succ = true;
    info.message = ' success';
elseif i == maxit
    info.succ = false;
    info.message = ' Maximun number of iteration has achieved!';
elseif ~conv
    info.succ = false;
    info.message = ' Global verification does not converge, result can be inaccurate.';
else
    info.succ = false;
    info.message = ' Horizontal search does not converge, result can be inaccurate.';
end

return
