function [alphat, betat, gammat, isred, itv] = ...
            rpsa_v_reduce(A, B, epsln, alpha, beta, gamma, itv0, htol)
%   Interval reduction applied to vertical line with
%   real part alpha, and intervals itv0 (each row being an interval).
%   Return the rightmost point if successful,
%   otherwise, return the input alpha, beta, and gamma.
%
%   itv records the reduced interval
%
%   isred = 1 if reduction is successful. Otherwise, isred = 0.

%   POSSIBLE IMPROVEMENT : BREAK WHEN THE REDUCED INTERVAL TOO SMALL

%   Revised: Jun. 13, 2016.

vtol = eps;
isred = 0; 

alphat = alpha; betat = beta; gammat = gamma; 
itv = [-inf, inf, -inf, inf];

for jj = 1 : size(itv0,1) 
    bl = itv0(jj,1); % upper and lower bound
    bu = itv0(jj,2);

    % reduce the interval
    [bl, bu, ~, info] = single_interval_reduce(A, B, alpha, bl, bu, epsln, vtol);

    if info == 4 % lower bound inside pseudospectrum (possible real line point)
        cc = bl;
        [alphatt, gammatt, ~] = rpsa_h_search(A, B, alpha, cc, epsln, htol);
        isred = 1;

    elseif info == 1 % reduced interval found, search use midpoint
        cc = 0.5 * (bl + bu);
        [alphatt, gammatt, ~] = rpsa_h_search(A, B, alpha, cc, epsln, htol);
        isred = 1;

    else % reduced interval is empty, fail
        alphatt = alpha;
    end

    if alphatt > alphat % find larger real part, update
        alphat = alphatt; 
        betat = cc; 
        gammat = gammatt;
        itv = [itv0(jj,:), bl, bu];
        return
    end
end

return;



% ========================================================================
% AUXILIARY FUNCTION
% ========================================================================

function [bl, bu, itr, info] = single_interval_reduce(A, B, alpha, bl, bu, epsln, tol)
%   Save guard interval reduction using vertical monotonic reduction
%
%   INPUT:
%       alpha   vertical line real part
%       bl, bu  candidate interval
%       epsln   perturbation level
%       tol     the (relative) tolerance s.t. 
%               |eval_mu(alpha + j*beta0) - epsln|  < epsln*tol
%               default by 1.0E-6
%
%   OUTPUT:
%       bl, bu  lower and upper bound
%               beta = -inf, if no intersection was found. 
%       itr     number of iterations.
%       info    information flag
%               info = 1    success
%               info = 2,3  no interval is found
%               info = 4    use bl as new search interval
%

if nargin < 7, tol = 1.0E-6; end

maxit = 30;
info = 1;

% Phase I: upper bound reduction

for i = 1 : maxit
    % update gamma
    [epsln_relax, gamma] = eval_mu(A,B,alpha,bu,tol);
    if epsln_relax - epsln < epsln*tol % already on the boundary
        break
    end
    
    % update bu
    bu0 = bu; 
    [reg,~] = superset_eig_imag(A, B, alpha, bu, gamma, epsln);
    bu = max( reg( reg< (bu + abs(bu0)*1.0E-6) ) );
    if isempty(bu) || bu < bl   % upper bound out of interval,
                                % interval reduction failed
        bl = -inf;
        bu = inf;
        itr = [i,0]; 
        info = 2;
        return
    end

    if bu0 - bu < abs(bu0)*tol  % update in bu too small, 
                                % convergence achieved
        break
    end
end


% Phase II: lower bound reduction

if bl == 0 % so we were not able to do `reduction' from below
    c =0.5 * ( bu + bl );
    [epsln_relax, gamma] = eval_mu(A,B,alpha,c,tol);
    bl = c;
    if epsln_relax <= epsln + epsln*10*tol % already inside the RPS
        info = 4;
        itr =[i,0];
        return
    end
end

% bl outside the RPS, do reduction...
for j = 1 : maxit
    % update gamma
    [epsln_relax, gamma] = eval_mu(A,B,alpha,bl,tol);
    if epsln_relax - epsln < epsln*10*tol
        % already on the boundary
        break
    end
    
    % update bl
    bl0 = bl; 
    [reg,~] = superset_eig_imag(A, B, alpha, bl, gamma, epsln);
    bl = min( reg( reg>bl - abs(bl0)*1.0E-6 ) );
    if isempty(bl) || bl > bu   % lower bound out of interval,
                                % interval reduction failed
        bl = -inf;
        itr = [i,j]; 
        info = 3;
        return
    end
    if bl - bl0 < abs(bl0)*tol  % update in bu too small, 
                                % convergence achieved
        break
    end
end
itr =[i,j];
return
% END
