function [s, v, u] = second_smallest_sv(A, B, alpha, beta, gamma, nsvds)
% s = second_smallest_sv(A, B, alpha, beta, gamma) solves the second 
%   smallest singular value of
%       [A - alpha * B, -beta * gamma * B]
%       [beta / gamma * B A - alpha * B  ]
%
%   [s, v, u] = sssig(A, B, alpha, beta, gamma) also returns the singular 
%   vectors corresponding to s.
%
%   Valid for both dense and sparse matrices.
%
%   POSSIBLE IMPROVEMENTS: 
%       1.  Return the singular vector space for multiple singular value
%       2.  Simplify computation in beta==0 case 

%   Revised: Jun. 13, 2016.

k = size(A,2);
n = size(A,1);

if nargin < 6 || isempty(nsvds)
    nsvds = 0;
end

if ~issparse(B)
    svd_solver = @(X) svd(X, 0); % use dense svd
else
    v = version;    % check MATLAB version
    if v(1)~='9'    % MATLAB 2015b, and earlier versions.
        svd_solver = @(X) svds(X, max([nsvds,6]), 0); 
    else            % MATLAB 2016a
        svd_solver = @(X) svds(X, max([nsvds,2]), 'smallest'); 
    end
end

% The `singular' case gamma = 0 (for safe guarding purpose).
if gamma == 0
    if issparse(B)
        warning('Evaluation of g(alpha, beta, gamma) at gammm = 0 is not allowed for sparse case');
        s = 0;
        return;
    end
    X = A + alpha * B; Y = beta * B;
    U = null(Y');
    H = [U' * X; zeros(n,k)];
    S = svd_solver(H);
    s = S(k-1) ;
    return;
end

H = [A - alpha * B,     -beta * gamma * B; 
    beta / gamma * B,   A - alpha * B];

if nargout < 2
    S = svd_solver(H);
    s = S(end-1);
else
    [U, S, V] = svd_solver(H);
    s = S(end-1, end-1);
    v = V(:, end-1);
    u = U(:, end-1);
end


return
