function [beta1, itv] = superset_eig_imag(A, B, alpha, beta, gamma, epsln)
%   Do the vertical search, find intersection points 
%   lambda = alpha + beta1*i with the superset Lambda_{epsln,gamma}(A,B),
%   by solving the Hamiltonian/Skew-Hamiltonian eigenvalue problem.
%   Due to the conjugate symmetry of eigenvalues, only beta1 with 
%   positive sign are returned.

%
%   NOTE: beta input is a dummy variable.
%
%   OUTPUT:
%   each elements beta1 records the intersections. 
%
%   POSSIBLE IMPROVEMENTS:
%       1.  real eigenvalue check threshold set based on backward
%           error of qz algorithm.
%       2.  sigular value order checking can be replaced by 
%           some simpler strategy
%       3.  maximizing interval ordering strategy can be improved.

%   Revised: Jun. 13, 2016.

tol1 = 1.0E-6;  % real eigenvalue, and singular value order checking
                % threshold (can be refined)
tol2 = 1.0E-6; 

n = size(A,1);
k = size(A,2);

% compute the eigenvalues

if n == k && isequal(B, eye(n))
    % square case
    H =[zeros(n,n), A'-alpha*B', epsln/gamma*eye(n), zeros(n,n);
        A'- alpha*B',zeros(n,n), zeros(n,n), epsln*gamma*eye(n);
        -epsln/gamma*eye(n), zeros(n,n), zeros(n,n), -A+alpha*B;
        zeros(n,n), -epsln*gamma*eye(n), -A+alpha*B, zeros(n,n)];
    eg = eig(H);
else
    % non-square case
    H = [epsln/gamma*eye(n), zeros(n,n), zeros(n,k), A-alpha*B ;
        zeros(n,n), epsln*gamma*eye(n), A-alpha*B, zeros(n,k);
       zeros(k,n), A'-alpha*B', epsln/gamma*eye(k), zeros(k,k);
        A'- alpha*B',zeros(k,n), zeros(k,k), epsln*gamma*eye(k)];
    G = blkdiag(B',B',-B, -B);
    G = [G(2*k+1:end,:); G(1:2*k,:)];
    eg = eig(H,G);
end

eg = eg*sqrt(-1); % real imag conversion

% find the imag eigenvalues
% reg = real(eg(angle(eg)<tol1));
reg = real( eg(   angle(abs(real(eg)) + sqrt(-1)*abs(imag(eg))) <tol1    ));
l1 = length(reg);

reg = reg(reg > 0);         %remove eigs of negative sign
reg(reg==inf) = [];         %remove inf eigs
reg = sort(reg, 'descend');

% possibly zero eigenvalue not detected
if l1 > 2*length(reg)
    reg = [reg;0];
end

% remove the first eig, since it must correspond to the
% smallest singular value, instead of the second smallest.
% if ~isempty(reg), reg(1) = [];end;

% check order of singular value for reg
% and delete possible common intersections by the smallest 
% and second smallest singular value
sg = 0;
for i =1:length(reg)
    beta = reg(i);
    HH = [A-alpha*B, -beta*gamma*B; beta/gamma*B, A-alpha*B];
    S = svd(HH,0);
    if abs( S(2*k-1) - epsln ) > tol1*epsln
        % not the second smallest singular value
        reg(i) = -inf;
        sg = 0;
        continue;
    elseif abs( S(2*k-1) - S(2*k)) < tol2^2*S(1)
        % smallest and second smallest possibly overlap 
        % remove one of them
        if sg == 1
           reg(i) = -inf;
           sg = 0;
        else
           sg = 1;
        end 
    end
end
reg(reg == -inf) = [];

% generate maximizing interval
% checking mid point for robustness (this can be removed)

itv = []; 
i = 1;
while i < length(reg)
    beta = (reg(i) + reg(i+1))/2;
    s = second_smallest_sv(A, B, alpha, beta, gamma);
    if s  < (1+tol1)*epsln
        itv = [itv; reg(i+1), reg(i)];
    end
    i = i + 1;
end

% check possible zero intersection
if ~isempty(reg) && reg(end) ~= 0
    s = second_smallest_sv(A, B, alpha, 0, gamma);
    if s < (1+tol1)*epsln 
        itv = [itv; 0, 0];
    end
end

beta1 = reg;
