function reg = superset_eig_real(A, B, alpha, beta, gamma, epsln)
%   Compute the rightmost lambda = alpha1 + beta*i in the superset 
%   Lambda_{epsln,gamma}(A,B), by solving the generalized eigenvalue 
%   problem.
%
%   NOTE: alpha1 is guaranteed to be larger than alpha
%
%   INPUT:
%       (alpha, beta):  starting searching point
%                       NOTE: alpha input is a dummy variable.
%       gamma, epsln, A, B: coefficients.
%
%   OUTPUT:
%       reg      intersections larger than alpha
%
%   POSSIBLE IMPROVEMENTS:
%       1.  real eigenvalue check threshold set... based on backward
%           error of qz algorithm
%       2.  singular value order checking can be replaced by 
%           some simpler strategy
%       3.  Better way to detect identity matrix than isequal

%   Revised: Jun. 13, 2016.

n = size(A,1);
k = size(A,2);

tol1 = 1.0E-6;  % real eigenvalue, and singular value order checking
                % threshold (can be refined)

% compute the eigenvalues

if isequal(B, eye(k)) % square case
    H = [A', beta / gamma * B, -epsln * B, zeros(k,k);
        -beta * gamma * B', A', zeros(k,k), -epsln * B; 
        -epsln * B, zeros(n,n), A, -beta * gamma * B;
        zeros(n,n), -epsln * B, beta / gamma * B, A];
    eg = eig(H);
else % rectangular case
    H = [-epsln * eye(n), zeros(n,n), A, -beta * gamma * B;
        zeros(n,n), -epsln * eye(n), beta / gamma * B, A;
        A', beta / gamma * B', -epsln * eye(k), zeros(k,k);
        -beta * gamma * B', A', zeros(k,k), -epsln * eye(k)];
    G = blkdiag(B', B', B, B); 
    G = [G(2 * k + 1 : end,:); G(1 : 2 * k,:)];
    eg = eig(H, G);
end

% find the real eigenvalues larger than alpha
reg = real( eg( abs( imag(eg) ./ real(eg)) < tol1 ) );
reg = reg(reg >= alpha);  %remove eigs smaller than alpha
reg(reg == inf) = [];     %remove inf eigs

% check order of singular value for reg
for i = 1 : length(reg)
    alpha0 = reg(i);
    HH = [A - alpha0 * B, -beta * gamma * B;
          beta / gamma * B, A - alpha0 * B];
    S = svd(HH);
    if abs(S(2*k-1) - epsln) > tol1 * epsln
        % not the second smallest eig
        reg(i) = -inf;
    end
end

% alpha1 = max([reg;alpha]);
reg(reg == -inf) = [];

return
% END
